@@ -4,17 +4,26 @@ module Cinch
   # @api private
   # @since 1.1.0
   module ModeParser
+    ErrEmptyString = "Empty mode string"
+    MalformedError = Struct.new(:modes)
+    EmptySequenceError = Struct.new(:modes)
+    NotEnoughParametersError = Struct.new(:op)
+    TooManyParametersError = Struct.new(:modes, :params)
+
     # @param [String] modes The mode string as sent by the server
     # @param [Array<String>] params Parameters belonging to the modes
     # @param [Hash{:add, :remove => Array<String>}] param_modes
     #   A mapping describing which modes require parameters
+    # @return [(Array<(Symbol<:add, :remove>, String<char>, String<param>), foo)]
     def self.parse_modes(modes, params, param_modes = {})
       if modes.size == 0
-        raise Exceptions::InvalidModeString, 'Empty mode string'
+        return nil, ErrEmptyString
+        # raise Exceptions::InvalidModeString, 'Empty mode string'
       end
 
       if modes[0] !~ /[+-]/
-        raise Exceptions::InvalidModeString, "Malformed modes string: %s" % modes
+        return nil, MalformedError.new(modes)
+        # raise Exceptions::InvalidModeString, "Malformed modes string: %s" % modes
       end
 
       changes = []
@@ -25,7 +34,8 @@ module Cinch
       modes.each_char do |ch|
         if ch =~ /[+-]/
           if count == 0
-            raise Exceptions::InvalidModeString, 'Empty mode sequence: %s' % modes
+            return changes, EmptySequenceError.new(modes)
+            # raise Exceptions::InvalidModeString, 'Empty mode sequence: %s' % modes
           end
 
           direction = case ch
@@ -41,7 +51,8 @@ module Cinch
             if params.size > 0
               param = params.shift
             else
-              raise Exceptions::InvalidModeString, 'Not enough parameters: %s' % ch.inspect
+              return changes, NotEnoughParametersError.new(ch)
+              # raise Exceptions::InvalidModeString, 'Not enough parameters: %s' % ch.inspect
             end
           end
           changes << [direction, ch, param]
@@ -50,14 +61,16 @@ module Cinch
       end
 
       if params.size > 0
-        raise Exceptions::InvalidModeString, 'Too many parameters: %s %s' % [modes, params]
+        return changes, TooManyParametersError.new(modes, params)
+        # raise Exceptions::InvalidModeString, 'Too many parameters: %s %s' % [modes, params]
       end
 
       if count == 0
-        raise Exceptions::InvalidModeString, 'Empty mode sequence: %s' % modes
+        return changes, EmptySequenceError.new(modes)
+        # raise Exceptions::InvalidModeString, 'Empty mode sequence: %s' % modes
       end
 
-      return changes
+      return changes, nil
     end
   end
 end
